@@ -16,13 +16,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-embed-persist.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-embed-persist.c 7964 2008-02-17 23:23:36Z xan $
  */
 
 #include "config.h"
 
 #include "ephy-embed-persist.h"
-#include "mozilla-embed-persist.h"
 #include "ephy-embed-type-builtins.h"
 #include "ephy-debug.h"
 
@@ -61,35 +60,7 @@ struct _EphyEmbedPersistPrivate
 static void	ephy_embed_persist_class_init	(EphyEmbedPersistClass *klass);
 static void	ephy_embed_persist_init		(EphyEmbedPersist *ges);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_embed_persist_get_type (void)
-{
-       static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyEmbedPersistClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_embed_persist_class_init,
-			NULL, /* class_finalize */
-			NULL, /* class_data */
-			sizeof (EphyEmbedPersist),
-			0,    /* n_preallocs */
-			(GInstanceInitFunc) ephy_embed_persist_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyEmbedPersist",
-					       &our_info, G_TYPE_FLAG_ABSTRACT);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyEmbedPersist, ephy_embed_persist, G_TYPE_OBJECT)
 
 /**
  * ephy_embed_persist_set_dest:
@@ -523,7 +494,7 @@ ephy_embed_persist_finalize (GObject *object)
 
 	LOG ("EphyEmbedPersist finalised %p", object);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_embed_persist_parent_class)->finalize (object);
 }
 
 static void
@@ -531,8 +502,6 @@ ephy_embed_persist_class_init (EphyEmbedPersistClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_embed_persist_finalize;
 	object_class->set_property = ephy_embed_persist_set_property;
 	object_class->get_property = ephy_embed_persist_get_property;
