@@ -15,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-encodings.c 6741 2006-12-17 03:06:15Z bjornl $
+ *  $Id: ephy-encodings.c 7964 2008-02-17 23:23:36Z xan $
  */
 
 #include "config.h"
@@ -187,35 +187,7 @@ enum
 static void ephy_encodings_class_init	(EphyEncodingsClass *klass);
 static void ephy_encodings_init		(EphyEncodings *ma);
 
-static GObjectClass *parent_class = NULL;
-
-GType
-ephy_encodings_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyEncodingsClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_encodings_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyEncodings),
-			0,
-			(GInstanceInitFunc) ephy_encodings_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyEncodings",
-					        &our_info, 0);
-	}
-
-	return type;
-}
+G_DEFINE_TYPE (EphyEncodings, ephy_encodings, G_TYPE_OBJECT)
 
 static void
 ephy_encodings_finalize (GObject *object)
@@ -235,7 +207,7 @@ ephy_encodings_finalize (GObject *object)
 
 	LOG ("EphyEncodings finalised");
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_encodings_parent_class)->finalize (object);
 }
 
 static void
@@ -243,8 +215,6 @@ ephy_encodings_class_init (EphyEncodingsClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_encodings_finalize;
 
 	g_type_class_add_private (object_class, sizeof (EphyEncodingsPrivate));
