@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2000-2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004 Christian Persch
@@ -14,9 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.131 2006/10/24 21:17:56 jframeau Exp $
  */
 
 #include "mozilla-config.h"
@@ -33,18 +33,20 @@
 #include <nsIWebProgressListener.h>
 #include <nsMemory.h>
 
+#include <glib/gi18n.h>
+
 #include "EphyBrowser.h"
 #include "EphyUtils.h"
 #include "EventContext.h"
 
 #include "ephy-command-manager.h"
 #include "ephy-debug.h"
+#include "mozilla-embed.h"
+#include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
-#include "ephy-string.h"
 #include "mozilla-embed-event.h"
-
-#include "mozilla-embed.h"
+#include "mozilla-history-item.h"
 
 static void	mozilla_embed_class_init	(MozillaEmbedClass *klass);
 static void	mozilla_embed_init		(MozillaEmbed *gs);
@@ -62,7 +64,7 @@ static void mozilla_embed_net_state_all_cb	(GtkMozEmbed *embed,
 static gboolean mozilla_embed_dom_mouse_click_cb(GtkMozEmbed *embed,
 						 gpointer dom_event,
 						 MozillaEmbed *membed);
-static gboolean mozilla_embed_dom_mouse_down_cb	(GtkMozEmbed *embed,
+static gboolean mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed,
 						 gpointer dom_event, 
 						 MozillaEmbed *membed);
 static gboolean mozilla_embed_dom_key_press_cb	(GtkMozEmbed *embed,
@@ -76,6 +78,21 @@ static void mozilla_embed_security_change_cb	(GtkMozEmbed *embed,
 						 gpointer request,
 						 PRUint32 state,
 						 MozillaEmbed *membed);
+static void mozilla_embed_title_change_cb	(GtkMozEmbed *embed,
+						 MozillaEmbed *membed);
+static void mozilla_embed_link_message_cb	(GtkMozEmbed *embed,
+						 MozillaEmbed *membed);
+static void mozilla_embed_visibility_cb		(GtkMozEmbed *embed,
+						 gboolean visibility,
+						 MozillaEmbed *membed);
+static gboolean mozilla_embed_open_uri_cb	(GtkMozEmbed *embed,
+						 const char *uri,
+						 MozillaEmbed *membed);
+static void mozilla_embed_size_to_cb		(GtkMozEmbed *embed,
+						 int width,
+						 int height,
+						 MozillaEmbed *membed);
+
 static EphyEmbedSecurityLevel mozilla_embed_security_level (PRUint32 state);
 
 #define MOZILLA_EMBED_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), MOZILLA_TYPE_EMBED, MozillaEmbedPrivate))
@@ -91,13 +108,14 @@ typedef enum
 struct MozillaEmbedPrivate
 {
 	EphyBrowser *browser;
+	GtkMozEmbed *moz_embed;
 	MozillaEmbedLoadState load_state;
+
+	guint idle_resize_handler;
 };
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
-static GObjectClass *parent_class = NULL;
-
 static void
 impl_manager_do_command (EphyCommandManager *manager,
 			 const char *command) 
@@ -115,7 +133,7 @@ impl_manager_can_do_command (EphyCommandManager *manager,
 	nsresult rv;
 	PRBool enabled;
 
-        rv = mpriv->browser->GetCommandState (command, &enabled);
+	rv = mpriv->browser->GetCommandState (command, &enabled);
 
 	return NS_SUCCEEDED (rv) ? enabled : FALSE;
 }
@@ -126,73 +144,13 @@ ephy_command_manager_iface_init (EphyCommandManagerIface *iface)
 	iface->do_command = impl_manager_do_command;
 	iface->can_do_command = impl_manager_can_do_command;
 }
+
+G_DEFINE_TYPE_WITH_CODE (MozillaEmbed, mozilla_embed, EPHY_TYPE_BASE_EMBED,
+			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_EMBED,
+						ephy_embed_iface_init)
+			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_COMMAND_MANAGER,
+						ephy_command_manager_iface_init))
 	
-GType 
-mozilla_embed_get_type (void)
-{
-        static GType type = 0;
-
-        if (G_UNLIKELY (type == 0))
-        {
-                const GTypeInfo our_info =
-                {
-                        sizeof (MozillaEmbedClass),
-                        NULL, /* base_init */
-                        NULL, /* base_finalize */
-                        (GClassInitFunc) mozilla_embed_class_init,
-                        NULL,
-                        NULL, /* class_data */
-                        sizeof (MozillaEmbed),
-                        0, /* n_preallocs */
-                        (GInstanceInitFunc) mozilla_embed_init
-                };
-
-		const GInterfaceInfo embed_info =
-		{
-			(GInterfaceInitFunc) ephy_embed_iface_init,
-        		NULL,
-        		NULL
-     		};
-
-		const GInterfaceInfo ephy_command_manager_info =
-		{
-			(GInterfaceInitFunc) ephy_command_manager_iface_init,
-        		NULL,
-        		NULL
-     		 };
-	
-                type = g_type_register_static (GTK_TYPE_MOZ_EMBED,
-					       "MozillaEmbed",
-					       &our_info, 
-					       (GTypeFlags)0);
-		g_type_add_interface_static (type,
-                                   	     EPHY_TYPE_EMBED,
-                                   	     &embed_info);
-		g_type_add_interface_static (type,
-                                   	     EPHY_TYPE_COMMAND_MANAGER,
-                                   	     &ephy_command_manager_info);
-        }
-
-        return type;
-}
-
-static void
-mozilla_embed_grab_focus (GtkWidget *widget)
-{
-	GtkWidget *child;
-
-	child = gtk_bin_get_child (GTK_BIN (widget));
-
-	if (child != NULL)
-	{
-		gtk_widget_grab_focus (child);
-	}
-	else
-	{
-		g_warning ("Need to realize the embed before grabbing focus!\n");
-	}
-}
-
 static void
 impl_close (EphyEmbed *embed) 
 {
@@ -206,11 +164,12 @@ mozilla_embed_realize (GtkWidget *widget)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (widget)->priv;
 
-	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+	GTK_WIDGET_CLASS (mozilla_embed_parent_class)->realize (widget);
 
+	gtk_widget_realize (GTK_WIDGET (mpriv->moz_embed));
 	/* Initialise our helper class */
 	nsresult rv;
-	rv = mpriv->browser->Init (GTK_MOZ_EMBED (widget));
+	rv = mpriv->browser->Init (EPHY_EMBED (widget));
 	if (NS_FAILED (rv))
 	{
 		g_warning ("EphyBrowser initialization failed for %p\n", widget);
@@ -227,57 +186,134 @@ mozilla_embed_constructor (GType type, guint n_construct_properties,
 	/* we depend on single because of mozilla initialization */
 	ephy_embed_shell_get_embed_single (embed_shell);
 
-	return parent_class->constructor (type, n_construct_properties,
-					  construct_params);
+	return G_OBJECT_CLASS (mozilla_embed_parent_class)->constructor (type, n_construct_properties,
+									 construct_params);
+}
+
+static void
+mozilla_embed_size_allocate (GtkWidget *widget,
+			     GtkAllocation *allocation)
+{
+	GtkWidget *child;
+	GtkAllocation invalid = { -1, -1, 1, 1 };
+
+	widget->allocation = *allocation;
+
+	child = GTK_BIN (widget)->child;
+	g_return_if_fail (child != NULL);
+
+	/* only resize if we're mapped (bug #128191),
+	 * or if this is the initial size-allocate (bug #156854).
+	 */
+	if (GTK_WIDGET_MAPPED (child) ||
+	    memcmp (&child->allocation, &invalid, sizeof (GtkAllocation)) == 0)
+	{
+		gtk_widget_size_allocate (child, allocation);
+	}
+}
+
+static void
+mozilla_embed_grab_focus (GtkWidget *widget)
+{
+	GtkWidget *child;
+
+	/* What we want to focus here is the MozContainer inside the
+	 * GtkMozEmbed, because the GtkMozEmbed itself does not gain focus.
+	 */
+	child = gtk_bin_get_child (GTK_BIN (MOZILLA_EMBED (widget)->priv->moz_embed));
+
+	if (child && GTK_WIDGET_REALIZED (child))
+		gtk_widget_grab_focus (child);
+	else
+		g_warning ("Need to realize the embed before grabbing focus!\n");
+}
+
+static void
+mozilla_embed_dispose (GObject *object)
+{
+	MozillaEmbedPrivate *priv = MOZILLA_EMBED (object)->priv;
+
+	if (priv->idle_resize_handler != 0)
+	{
+		g_source_remove (priv->idle_resize_handler);
+		priv->idle_resize_handler = 0;
+	}
+
+	G_OBJECT_CLASS (mozilla_embed_parent_class)->dispose (object);
 }
 
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
-     	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass); 
-     	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass); 
-
-	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass); 
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass); 
 
 	object_class->constructor = mozilla_embed_constructor;
 	object_class->finalize = mozilla_embed_finalize;
+	object_class->dispose = mozilla_embed_dispose;
 
 	gtk_object_class->destroy = mozilla_embed_destroy;
 
-	widget_class->grab_focus = mozilla_embed_grab_focus;
 	widget_class->realize = mozilla_embed_realize;
+	widget_class->size_allocate = mozilla_embed_size_allocate;
+	widget_class->grab_focus = mozilla_embed_grab_focus;
 
 	g_type_class_add_private (object_class, sizeof(MozillaEmbedPrivate));
 }
 
 static void
-mozilla_embed_init (MozillaEmbed *embed)
+mozilla_embed_init (MozillaEmbed *membed)
 {
-        embed->priv = MOZILLA_EMBED_GET_PRIVATE (embed);
-	embed->priv->browser = new EphyBrowser ();
+	MozillaEmbedPrivate *priv;
+	GtkWidget *embed;
+
+	membed->priv = MOZILLA_EMBED_GET_PRIVATE (membed);
+	priv = membed->priv;
+
+	embed = gtk_moz_embed_new ();
+	priv->moz_embed = GTK_MOZ_EMBED (embed);
+	gtk_container_add (GTK_CONTAINER (membed), embed);
+	gtk_widget_show (embed);
+
+	priv->browser = new EphyBrowser ();
 
 	g_signal_connect_object (embed, "location",
 				 G_CALLBACK (mozilla_embed_location_changed_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "net_state_all",
 				 G_CALLBACK (mozilla_embed_net_state_all_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "dom_mouse_click",
 				 G_CALLBACK (mozilla_embed_dom_mouse_click_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "dom_mouse_down",
 				 G_CALLBACK (mozilla_embed_dom_mouse_down_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "dom-key-press",
 				 G_CALLBACK (mozilla_embed_dom_key_press_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "new_window",
 				 G_CALLBACK (mozilla_embed_new_window_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
 	g_signal_connect_object (embed, "security_change",
 				 G_CALLBACK (mozilla_embed_security_change_cb),
-				 embed, (GConnectFlags) 0);
+				 membed, (GConnectFlags) 0);
+	g_signal_connect_object (embed, "title",
+				 G_CALLBACK (mozilla_embed_title_change_cb),
+				 membed, (GConnectFlags) 0);
+	g_signal_connect_object (embed, "link_message",
+				 G_CALLBACK (mozilla_embed_link_message_cb),
+				 membed, (GConnectFlags)0);
+	g_signal_connect_object (embed, "open_uri",
+				 G_CALLBACK (mozilla_embed_open_uri_cb),
+				 membed,(GConnectFlags) 0);
+	g_signal_connect_object (embed, "visibility",
+				 G_CALLBACK (mozilla_embed_visibility_cb),
+				 membed, (GConnectFlags) 0);
+	g_signal_connect_object (embed, "size_to",
+				 G_CALLBACK (mozilla_embed_size_to_cb),
+				 membed, (GConnectFlags) 0);
 }
 
 gpointer
@@ -298,7 +334,7 @@ mozilla_embed_destroy (GtkObject *object)
 		embed->priv->browser->Destroy();
 	}
 	
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	GTK_OBJECT_CLASS (mozilla_embed_parent_class)->destroy (object);
 }
 
 static void
@@ -308,27 +344,28 @@ mozilla_embed_finalize (GObject *object)
 
 	if (embed->priv->browser)
 	{
-        	delete embed->priv->browser;
-	       	embed->priv->browser = nsnull;
+		delete embed->priv->browser;
+		embed->priv->browser = nsnull;
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (mozilla_embed_parent_class)->finalize (object);
 
 	g_object_unref (embed_shell);
 }
 
 static void
 impl_load_url (EphyEmbed *embed, 
-               const char *url)
+	       const char *url)
 {
-        gtk_moz_embed_load_url (GTK_MOZ_EMBED(embed), url);
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	gtk_moz_embed_load_url (moz_embed, url);
 }
 
 static char * impl_get_location (EphyEmbed *embed, gboolean toplevel);
 
 static void
 impl_load (EphyEmbed *embed, 
-           const char *url,
+	   const char *url,
 	   EphyEmbedLoadFlags flags,
 	   EphyEmbed *preview_embed)
 {
@@ -357,26 +394,29 @@ impl_load (EphyEmbed *embed,
 	else
 #endif /* HAVE_GECKO_1_8_1 */
 	{
-		browser->LoadURI (url, nsIWebNavigation::LOAD_FLAGS_NONE, uri);	
+		browser->LoadURI (url, nsIWebNavigation::LOAD_FLAGS_NONE, uri); 
 	}
 }
 
 static void
 impl_stop_load (EphyEmbed *embed)
 {
-	gtk_moz_embed_stop_load (GTK_MOZ_EMBED(embed));	
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	gtk_moz_embed_stop_load (moz_embed);	
 }
 
 static gboolean
 impl_can_go_back (EphyEmbed *embed)
 {
-	return gtk_moz_embed_can_go_back (GTK_MOZ_EMBED(embed));
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	return gtk_moz_embed_can_go_back (moz_embed);
 }
 
 static gboolean
 impl_can_go_forward (EphyEmbed *embed)
 {
-	return gtk_moz_embed_can_go_forward (GTK_MOZ_EMBED(embed));
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	return gtk_moz_embed_can_go_forward (moz_embed);
 }
 
 static gboolean
@@ -384,14 +424,24 @@ mozilla_embed_get_uri_parent (MozillaEmbed *membed,
 			      const char *aUri,
 			      nsCString &aParent)
 {
-        nsresult rv;
+	nsresult rv;
 	nsCString encoding;
 	rv = membed->priv->browser->GetEncoding (encoding);
 	if (NS_FAILED (rv)) return FALSE;
+	
+	/* Check for HTML anchors */
+	nsCOMPtr<nsIURI> uri;
+	const char *anchor = strrchr (aUri, '#');
+	if (anchor)
+	{
+		aParent.Assign (aUri);
+		aParent.SetLength (anchor - aUri);
 
-        nsCOMPtr<nsIURI> uri;
-        rv = EphyUtils::NewURI (getter_AddRefs(uri), nsCString(aUri), encoding.get());
-        if (NS_FAILED(rv) || !uri) return FALSE;
+		return TRUE;
+	}
+	
+	rv = EphyUtils::NewURI (getter_AddRefs(uri), nsCString(aUri), encoding.get());
+	if (NS_FAILED(rv) || !uri) return FALSE;
 
 	/* Don't support going 'up' with chrome url's, mozilla handily
 	 * fixes them up for us, so it doesn't work properly, see
@@ -468,13 +518,15 @@ impl_get_go_up_list (EphyEmbed *embed)
 static void
 impl_go_back (EphyEmbed *embed)
 {
-	gtk_moz_embed_go_back (GTK_MOZ_EMBED(embed));
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	gtk_moz_embed_go_back (moz_embed);
 }
 		
 static void
 impl_go_forward (EphyEmbed *embed)
 {
-	gtk_moz_embed_go_forward (GTK_MOZ_EMBED(embed));
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	gtk_moz_embed_go_forward (moz_embed);
 }
 
 static void
@@ -496,27 +548,16 @@ impl_go_up (EphyEmbed *embed)
 	ephy_embed_load_url (embed, parent_uri.get ());
 }
 
-static char *
-impl_get_title (EphyEmbed *embed)
-{
-	return gtk_moz_embed_get_title (GTK_MOZ_EMBED (embed));
-}
-
-static char *
-impl_get_link_message (EphyEmbed *embed)
-{
-	return gtk_moz_embed_get_link_message (GTK_MOZ_EMBED (embed));
-}
-
 static char *
 impl_get_js_status (EphyEmbed *embed)
 {
-	return gtk_moz_embed_get_js_status (GTK_MOZ_EMBED (embed));
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
+	return gtk_moz_embed_get_js_status (moz_embed);
 }
 
 static char *
 impl_get_location (EphyEmbed *embed, 
-                   gboolean toplevel)
+		   gboolean toplevel)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
 	nsresult rv;
@@ -551,8 +592,9 @@ impl_get_location (EphyEmbed *embed,
 
 static void
 impl_reload (EphyEmbed *embed, 
-             gboolean force)
+	     gboolean force)
 {
+	GtkMozEmbed *moz_embed = MOZILLA_EMBED (embed)->priv->moz_embed;
 	guint32 mflags = GTK_MOZ_EMBED_FLAG_RELOADNORMAL;
 
 	if (force)
@@ -560,12 +602,12 @@ impl_reload (EphyEmbed *embed,
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE;
 	}
 
-	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed), mflags);
+	gtk_moz_embed_reload (moz_embed, mflags);
 }
 
 static void
 impl_set_zoom (EphyEmbed *embed, 
-               float zoom) 
+	       float zoom) 
 {
 	EphyBrowser *browser;
 	nsresult rv;
@@ -640,43 +682,6 @@ impl_shistory_n_items (EphyEmbed *embed)
 	return NS_SUCCEEDED(rv) ? count : 0;
 }
 
-static void
-impl_shistory_get_nth (EphyEmbed *embed, 
-                       int nth,
-                       gboolean is_relative,
-                       char **aUrl,
-                       char **aTitle)
-{
-	nsresult rv;
-        nsCString url;
-	PRUnichar *title;
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-
-	if (is_relative)
-	{
-		nth += ephy_embed_shistory_get_pos (embed);
-	}
-	
-        rv = mpriv->browser->GetSHUrlAtIndex(nth, url);
-
-        *aUrl = (NS_SUCCEEDED (rv) && url.Length()) ? g_strdup(url.get()) : NULL;
-
-	rv = mpriv->browser->GetSHTitleAtIndex(nth, &title);
-
-	if (title)
-	{
-		nsCString cTitle;
-		NS_UTF16ToCString (nsString(title),
-				   NS_CSTRING_ENCODING_UTF8, cTitle);
-		*aTitle = g_strdup (cTitle.get());
-		nsMemory::Free (title);
-	}
-	else
-	{
-		*aTitle = NULL;
-	}
-}
-
 static int
 impl_shistory_get_pos (EphyEmbed *embed)
 {
@@ -689,15 +694,6 @@ impl_shistory_get_pos (EphyEmbed *embed)
 	return NS_SUCCEEDED(rv) ? index : 0;
 }
 
-static void
-impl_shistory_go_nth (EphyEmbed *embed, 
-                      int nth)
-{
-	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED(embed)->priv;
-
-	mpriv->browser->GoToHistoryIndex (nth);
-}
-
 static void
 impl_shistory_copy (EphyEmbed *source,
 		    EphyEmbed *dest,
@@ -714,8 +710,8 @@ impl_shistory_copy (EphyEmbed *source,
 
 static void
 impl_get_security_level (EphyEmbed *embed,
-                         EphyEmbedSecurityLevel *level,
-                         char **description)
+			 EphyEmbedSecurityLevel *level,
+			 char **description)
 {
 	MozillaEmbedPrivate *mpriv = MOZILLA_EMBED (embed)->priv;
 
@@ -798,7 +794,7 @@ impl_set_encoding (EphyEmbed *embed,
 		if (NS_FAILED (rv)) return;
 	}
 
-	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
+	gtk_moz_embed_reload (mpriv->moz_embed,
 			      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
 }
 
@@ -848,21 +844,117 @@ impl_has_modified_forms (EphyEmbed *embed)
 	return NS_SUCCEEDED (rv) ? modified : FALSE;
 }
 
+static EphyHistoryItem*
+mozilla_get_nth_history_item (MozillaEmbed *embed,
+			      gboolean is_relative,
+			      int nth)
+{
+	EphyHistoryItem *item = NULL;
+
+	if (is_relative)
+	{
+		nth += impl_shistory_get_pos (EPHY_EMBED (embed));
+	}
+	
+	if (nth >= 0 && nth <= impl_shistory_n_items (EPHY_EMBED (embed)))
+	{
+		item = (EphyHistoryItem*)mozilla_history_item_new (embed, nth);
+	}
+
+	return item;
+}
+
+static GList*
+mozilla_construct_history_list (MozillaEmbed *embed, int start, int end)
+{
+	GList *list = NULL;
+	EphyHistoryItem *item;
+
+	while (start != end)
+	{
+		item = mozilla_get_nth_history_item (embed, FALSE, start);
+		list = g_list_prepend (list, item);
+
+		if (start < end)
+			start++;
+		else
+			start--;
+	}
+
+	return g_list_reverse (list);
+}
+
+static GList*
+impl_get_backward_history (EphyEmbed *embed)
+{
+	MozillaEmbed *membed = MOZILLA_EMBED (embed);
+	int start = impl_shistory_get_pos (embed) + -1;
+
+	return mozilla_construct_history_list (membed, start, -1);
+}
+
+static GList*
+impl_get_forward_history (EphyEmbed *embed)
+{
+	MozillaEmbed *membed = MOZILLA_EMBED (embed);
+	int start = impl_shistory_get_pos (embed) + 1;
+	int end = impl_shistory_n_items (embed);
+
+	return mozilla_construct_history_list (membed, start, end);
+}
+
+static EphyHistoryItem*
+impl_get_next_history_item (EphyEmbed *embed)
+{
+	return mozilla_get_nth_history_item (MOZILLA_EMBED (embed),
+					     TRUE,
+					     +1);
+}
+
+static EphyHistoryItem*
+impl_get_previous_history_item (EphyEmbed *embed)
+{
+	return mozilla_get_nth_history_item (MOZILLA_EMBED (embed),
+					     TRUE,
+					     -1);
+}
+
+static void
+impl_go_to_history_item (EphyEmbed *embed, EphyHistoryItem *item)
+{
+	MozillaHistoryItem *mitem = MOZILLA_HISTORY_ITEM (item);
+
+	MOZILLA_EMBED (embed)->priv->browser->GoToHistoryIndex (mitem->nth);
+}
+
 static void
 mozilla_embed_location_changed_cb (GtkMozEmbed *embed, 
 				   MozillaEmbed *membed)
 {
 	char *location;
+	EphyBaseEmbed *bembed = EPHY_BASE_EMBED (membed);
 
 	location = gtk_moz_embed_get_location (embed);
-	g_signal_emit_by_name (membed, "ge_location", location);
+
+	ephy_base_embed_location_changed (bembed, location);
+
 	g_free (location);
 }
 
+static void
+mozilla_embed_link_message_cb	    (GtkMozEmbed *embed,
+				     MozillaEmbed *membed)
+{
+	char *link_message = gtk_moz_embed_get_link_message (embed);
+	ephy_base_embed_set_link_message (EPHY_BASE_EMBED (membed), link_message);
+	g_free (link_message);
+}
+
 static void
 update_load_state (MozillaEmbed *membed, gint state)
 {
 	MozillaEmbedPrivate *priv = membed->priv;
+	GtkMozEmbed *moz_embed = priv->moz_embed;
 
 	if (state & GTK_MOZ_EMBED_FLAG_IS_DOCUMENT &&
 	    state & (GTK_MOZ_EMBED_FLAG_START | GTK_MOZ_EMBED_FLAG_STOP))
@@ -877,8 +969,10 @@ update_load_state (MozillaEmbed *membed, gint state)
 		priv->load_state = MOZILLA_EMBED_LOAD_LOADING;
 
 		char *address;
-		address = gtk_moz_embed_get_location (GTK_MOZ_EMBED (membed));
-		g_signal_emit_by_name (membed, "ge-content-change", address);
+		address = gtk_moz_embed_get_location (moz_embed);
+		g_signal_emit_by_name (membed, "new-document-now", address);
+		ephy_base_embed_popups_manager_reset (EPHY_BASE_EMBED (membed));
+		ephy_base_embed_restore_zoom_level (EPHY_BASE_EMBED (membed), address);
 		g_free (address);
 	}
 
@@ -894,7 +988,7 @@ update_load_state (MozillaEmbed *membed, gint state)
 		}
 	}
 	else if (state & GTK_MOZ_EMBED_FLAG_START &&
-	         state & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
+		 state & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
 	{
 		if (priv->load_state == MOZILLA_EMBED_LOAD_REDIRECTING)
 		{
@@ -905,13 +999,14 @@ update_load_state (MozillaEmbed *membed, gint state)
 			priv->load_state = MOZILLA_EMBED_LOAD_LOADING;
 
 			char *address;
-			address = gtk_moz_embed_get_location (GTK_MOZ_EMBED (membed));
-			g_signal_emit_by_name (membed, "ge_content_change", address);
+			address = gtk_moz_embed_get_location (moz_embed);
+			ephy_base_embed_popups_manager_reset (EPHY_BASE_EMBED (membed));
+			ephy_base_embed_restore_zoom_level (EPHY_BASE_EMBED (membed), address);
 			g_free (address);
 		}
 	}
 	else if (state & GTK_MOZ_EMBED_FLAG_REDIRECTING &&
-	         priv->load_state == MOZILLA_EMBED_LOAD_STARTED)
+		 priv->load_state == MOZILLA_EMBED_LOAD_STARTED)
 	{
 		priv->load_state = MOZILLA_EMBED_LOAD_REDIRECTING;
 	}
@@ -919,7 +1014,7 @@ update_load_state (MozillaEmbed *membed, gint state)
 
 static void
 mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
-                                gint state, guint status, 
+				gint state, guint status, 
 				MozillaEmbed *membed)
 {
 	EphyEmbedNetState estate = EPHY_EMBED_STATE_UNKNOWN;
@@ -953,8 +1048,7 @@ mozilla_embed_net_state_all_cb (GtkMozEmbed *embed, const char *aURI,
 	}
 
 	update_load_state (membed, state);
-	
-	g_signal_emit_by_name (membed, "ge_net_state", aURI, /* FIXME: (gulong) */ estate);
+	ephy_base_embed_update_from_net_state (EPHY_BASE_EMBED (membed), aURI, (EphyEmbedNetState)estate);
 }
 
 static gboolean
@@ -975,10 +1069,10 @@ mozilla_embed_emit_mouse_signal (MozillaEmbed *embed,
 	nsCOMPtr<nsIDOMEvent> dev = do_QueryInterface (ev);
 	NS_ENSURE_TRUE (dev, FALSE);
 
-	info = mozilla_embed_event_new (NS_STATIC_CAST (gpointer, dev));
+	info = mozilla_embed_event_new (static_cast<gpointer>(dev));
 
 	event_context.Init (mpriv->browser);
-        rv = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
+	rv = event_context.GetMouseEventInfo (ev, MOZILLA_EMBED_EVENT (info));
 	if (NS_FAILED (rv))
 	{
 		g_object_unref (info);
@@ -1037,7 +1131,8 @@ mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed,
 
 	g_return_val_if_fail (GDK_KEY_PRESS == event->type, FALSE);
 
-	g_signal_emit_by_name (embed, "ge-search-key-press", event, &retval);
+	g_signal_emit_by_name (EPHY_EMBED (membed),
+			       "ge-search-key-press", event, &retval);
 
 	gdk_event_free (event);
 
@@ -1098,11 +1193,11 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 
 	g_assert (new_embed != NULL);
 
-	gtk_moz_embed_set_chrome_mask (GTK_MOZ_EMBED (new_embed), chrome);
+	gtk_moz_embed_set_chrome_mask (embed, chrome);
 
 	g_signal_emit_by_name (membed, "ge-new-window", new_embed);
 
-	*newEmbed = GTK_MOZ_EMBED (new_embed);
+	*newEmbed = GTK_MOZ_EMBED (gtk_bin_get_child (GTK_BIN (new_embed)));
 }
 
 static void
@@ -1111,8 +1206,168 @@ mozilla_embed_security_change_cb (GtkMozEmbed *embed,
 				  PRUint32 state,
 				  MozillaEmbed *membed)
 {
-	g_signal_emit_by_name (membed, "ge_security_change",
-			       mozilla_embed_security_level (state));
+	ephy_base_embed_set_security_level (EPHY_BASE_EMBED (membed),
+					    mozilla_embed_security_level (state));
+}
+
+static void
+mozilla_embed_title_change_cb	    (GtkMozEmbed *embed,
+				     MozillaEmbed *membed)
+{
+	GObject *object = G_OBJECT (embed);
+	char *title;
+
+	title = gtk_moz_embed_get_title (embed);
+
+	g_object_freeze_notify (object);
+
+	ephy_base_embed_set_title (EPHY_BASE_EMBED (membed), title);
+	ephy_base_embed_set_loading_title (EPHY_BASE_EMBED (membed),
+					   title, FALSE);
+
+	g_free (title);
+
+	g_object_thaw_notify (object);
+}
+
+static gboolean
+mozilla_embed_open_uri_cb (GtkMozEmbed *embed,
+			   const char *uri,
+			   MozillaEmbed *membed)
+{
+	/* Set the address here if we have a blank page.
+	 * See bug #147840.
+	 */
+	if (ephy_embed_get_is_blank (EPHY_EMBED (membed)))
+	{
+		EphyBaseEmbed *bembed = EPHY_BASE_EMBED (membed);
+
+
+		ephy_base_embed_set_address (bembed, uri);
+		ephy_base_embed_set_loading_title (bembed, uri, TRUE);
+	}
+
+	/* allow load to proceed */
+	return FALSE;
+}
+
+static void
+mozilla_embed_visibility_cb (GtkMozEmbed *embed,
+			     gboolean visibility,
+			     MozillaEmbed *membed)
+{
+  ephy_base_embed_set_visibility (EPHY_BASE_EMBED (membed), visibility);
+}
+
+static gboolean
+let_me_resize_hack (MozillaEmbed *embed)
+{
+	GtkWidget *window;
+	MozillaEmbedPrivate *priv = embed->priv;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
+
+	if (window && GTK_WIDGET_TOPLEVEL (window))
+	  gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
+
+	priv->idle_resize_handler = 0;
+
+	return FALSE;
+}
+
+static void
+ephy_tab_set_size (MozillaEmbed *embed,
+		   int width,
+		   int height)
+{
+	MozillaEmbedPrivate *priv = embed->priv;
+	GtkWidget *widget = GTK_WIDGET (embed);
+	GtkAllocation allocation;
+
+	gtk_widget_set_size_request (widget, width, height);
+
+	/* HACK: When the web site changes both width and height,
+	 * we will first get a width change, then a height change,
+	 * without actually resizing the window in between (since
+	 * that happens only on idle).
+	 * If we don't set the allocation, GtkMozEmbed doesn't tell
+	 * mozilla the new width, so the height change sets the width
+	 * back to the old value!
+	 */
+	allocation.x = widget->allocation.x;
+	allocation.y = widget->allocation.y;
+	allocation.width = width;
+	allocation.height = height;
+	gtk_widget_size_allocate (widget, &allocation);
+
+	/* HACK: reset widget requisition after the container
+	 * has been resized. It appears to be the only way
+	 * to have the window sized according to embed
+	 * size correctly.
+	 */
+	if (priv->idle_resize_handler == 0)
+	{
+		priv->idle_resize_handler =
+			g_idle_add ((GSourceFunc) let_me_resize_hack, embed);
+	}
+}
+
+static void
+mozilla_embed_size_to_cb (GtkMozEmbed *embed,
+			  int width,
+			  int height,
+			  MozillaEmbed *membed)
+{
+	GtkWidget *widget;
+	EphyEmbedContainer *container;
+	MozillaEmbedPrivate *priv = MOZILLA_EMBED (membed)->priv;
+	GtkWidget *embed_widget = GTK_WIDGET (membed);
+	GdkScreen *screen;
+	GdkRectangle rect;
+	int monitor;
+	int ww, wh, ew, eh, dw, dh;
+
+	container = EPHY_EMBED_CONTAINER (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
+	if (! container || !GTK_WIDGET_TOPLEVEL (container))
+	  return;
+
+	/* FIXME: allow sizing also for non-popup single-tab windows? */
+	if (EPHY_EMBED (membed) != ephy_embed_container_get_active_child (container) ||
+	    ! ephy_embed_container_get_is_popup (container))
+	  return;
+
+	widget = GTK_WIDGET (container);
+
+	LOG ("mozilla_embed_size_to_cb window %p embed %p width %d height %d", widget, embed, width, height);
+
+	/* contrain size so that the window will be fully contained within the screen */
+	screen = gtk_widget_get_screen (widget);
+	monitor = gdk_screen_get_monitor_at_window (screen, widget->window);
+	gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+	/* FIXME: get and subtract the panel size */
+
+	gtk_window_get_size (GTK_WINDOW (widget), &ww, &wh);
+
+	ew = embed_widget->allocation.width;
+	eh = embed_widget->allocation.height;
+
+	/* This should approximate the chrome extent */
+	dw = ww - ew; dw = MAX (dw, 0); dw = MIN (dw, rect.width - 1);
+	dh = wh - eh; dh = MAX (dh, 0); dh = MIN (dh, rect.height - 1);
+
+	width = MIN (rect.width - dw, width);
+	height = MIN (rect.height - dh, height);
+
+	/* FIXME: move window if this will place it partially outside the screen rect? */
+
+	gtk_window_set_resizable (GTK_WINDOW (widget), FALSE);
+	ephy_tab_set_size (membed, width, height);
+
+	if (priv->idle_resize_handler == 0)
+	{
+		priv->idle_resize_handler =
+			g_idle_add ((GSourceFunc) let_me_resize_hack, membed);
+	}
 }
 
 static EphyEmbedSecurityLevel
@@ -1121,29 +1376,29 @@ mozilla_embed_security_level (PRUint32 state)
 	EphyEmbedSecurityLevel level;
 
 	switch (state)
-        {
-        case nsIWebProgressListener::STATE_IS_INSECURE:
-                level = EPHY_EMBED_STATE_IS_INSECURE;
-                break;
-        case nsIWebProgressListener::STATE_IS_BROKEN:
-                level = EPHY_EMBED_STATE_IS_BROKEN;
-                break;
-        case nsIWebProgressListener::STATE_IS_SECURE|
-             nsIWebProgressListener::STATE_SECURE_HIGH:
-                level = EPHY_EMBED_STATE_IS_SECURE_HIGH;
-                break;
-        case nsIWebProgressListener::STATE_IS_SECURE|
-             nsIWebProgressListener::STATE_SECURE_MED:
-                level = EPHY_EMBED_STATE_IS_SECURE_MED;
-                break;
-        case nsIWebProgressListener::STATE_IS_SECURE|
-             nsIWebProgressListener::STATE_SECURE_LOW:
-                level = EPHY_EMBED_STATE_IS_SECURE_LOW;
-                break;
-        default:
-                level = EPHY_EMBED_STATE_IS_UNKNOWN;
-                break;
-        }
+	{
+	case nsIWebProgressListener::STATE_IS_INSECURE:
+		level = EPHY_EMBED_STATE_IS_INSECURE;
+		break;
+	case nsIWebProgressListener::STATE_IS_BROKEN:
+		level = EPHY_EMBED_STATE_IS_BROKEN;
+		break;
+	case nsIWebProgressListener::STATE_IS_SECURE|
+	     nsIWebProgressListener::STATE_SECURE_HIGH:
+		level = EPHY_EMBED_STATE_IS_SECURE_HIGH;
+		break;
+	case nsIWebProgressListener::STATE_IS_SECURE|
+	     nsIWebProgressListener::STATE_SECURE_MED:
+		level = EPHY_EMBED_STATE_IS_SECURE_MED;
+		break;
+	case nsIWebProgressListener::STATE_IS_SECURE|
+	     nsIWebProgressListener::STATE_SECURE_LOW:
+		level = EPHY_EMBED_STATE_IS_SECURE_LOW;
+		break;
+	default:
+		level = EPHY_EMBED_STATE_IS_UNKNOWN;
+		break;
+	}
 	return level;
 }
 
@@ -1160,9 +1415,7 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->go_back = impl_go_back;
 	iface->go_forward = impl_go_forward;
 	iface->go_up = impl_go_up;
-	iface->get_title = impl_get_title;
 	iface->get_location = impl_get_location;
-	iface->get_link_message = impl_get_link_message;
 	iface->get_js_status = impl_get_js_status;
 	iface->reload = impl_reload;
 	iface->set_zoom = impl_set_zoom;
@@ -1170,12 +1423,7 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->scroll_lines = impl_scroll_lines;
 	iface->scroll_pages = impl_scroll_pages;
 	iface->scroll_pixels = impl_scroll_pixels;
-	iface->shistory_n_items = impl_shistory_n_items;
-	iface->shistory_get_nth = impl_shistory_get_nth;
-	iface->shistory_get_pos = impl_shistory_get_pos;
-	iface->shistory_go_nth = impl_shistory_go_nth;
 	iface->shistory_copy = impl_shistory_copy;
-	iface->get_security_level = impl_get_security_level;
 	iface->show_page_certificate = impl_show_page_certificate;
 	iface->close = impl_close;
 	iface->set_encoding = impl_set_encoding;
@@ -1186,6 +1434,12 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->print_preview_n_pages = impl_print_preview_n_pages;
 	iface->print_preview_navigate = impl_print_preview_navigate;
 	iface->has_modified_forms = impl_has_modified_forms;
+	iface->get_security_level = impl_get_security_level;
+	iface->get_backward_history = impl_get_backward_history;
+	iface->get_forward_history = impl_get_forward_history;
+	iface->get_next_history_item = impl_get_next_history_item;
+	iface->get_previous_history_item = impl_get_previous_history_item;
+	iface->go_to_history_item = impl_go_to_history_item;
 }
 
 static void
@@ -1213,9 +1467,9 @@ xul_new_window_cb (GtkMozEmbed *embed,
 		   guint chrome_mask,
 		   gpointer dummy)
 {
-        g_assert (chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME);
+	g_assert (chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME);
 
-        *retval = _mozilla_embed_new_xul_dialog ();
+	*retval = _mozilla_embed_new_xul_dialog ();
 }
 
 static void
@@ -1238,7 +1492,7 @@ _mozilla_embed_new_xul_dialog (void)
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	g_object_set_data_full (G_OBJECT (window), "EmbedShellRef",
-			        embed_shell,
+				embed_shell,
 				(GDestroyNotify) g_object_unref);
 	g_signal_connect_object (embed_shell, "prepare_close",
 				 G_CALLBACK (gtk_widget_destroy), window,
@@ -1266,3 +1520,11 @@ _mozilla_embed_new_xul_dialog (void)
 
 	return GTK_MOZ_EMBED (embed);
 }
+
+EphyBrowser*
+_mozilla_embed_get_browser (MozillaEmbed *embed)
+{
+	g_return_val_if_fail (MOZILLA_IS_EMBED (embed), NULL);
+
+	return embed->priv->browser;
+}
