@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-node-view.c,v 1.69 2006/12/17 03:06:14 bjornl Exp $
+ *  $Id: ephy-node-view.c 7762 2007-12-08 01:21:52Z cosimoc $
  */
 
 #include "config.h"
@@ -90,6 +90,7 @@ enum
 	NODE_ACTIVATED,
 	NODE_SELECTED,
 	NODE_DROPPED,
+	NODE_MIDDLE_CLICKED,
 	LAST_SIGNAL
 };
 
@@ -160,16 +161,21 @@ static EphyNode *
 get_node_from_path (EphyNodeView *view, GtkTreePath *path)
 {
 	EphyNode *node;
-	GtkTreeIter iter, iter2;
+	GtkTreeIter iter, iter2, iter3;
 
 	if (path == NULL) return NULL;
 
 	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
 	gtk_tree_model_sort_convert_iter_to_child_iter
 		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
+
+	if (iter2.stamp == 0) {
+		return NULL;
+	}
 	gtk_tree_model_filter_convert_iter_to_child_iter
-		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
-	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter);
+		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter3, &iter2);
+
+	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter3);
 
 	return node;
 }
@@ -568,6 +574,24 @@ path_toggled (GtkTreeModel *dummy_model, GtkTreePath *path,
 		       node, checked);
 }
 
+static EphyNode *
+process_middle_click (GtkTreePath *path,
+		      EphyNodeView *view)
+{
+	EphyNode *node;
+	GtkTreeIter iter, iter2;
+	
+	gtk_tree_model_get_iter (view->priv->sortmodel, &iter, path);
+	gtk_tree_model_sort_convert_iter_to_child_iter
+		(GTK_TREE_MODEL_SORT (view->priv->sortmodel), &iter2, &iter);
+	gtk_tree_model_filter_convert_iter_to_child_iter
+		(GTK_TREE_MODEL_FILTER (view->priv->filtermodel), &iter, &iter2);
+
+	node = ephy_tree_model_node_node_from_iter (view->priv->nodemodel, &iter);
+	
+	return node;
+}
+
 static gboolean
 ephy_node_view_key_press_cb (GtkTreeView *treeview,
 			     GdkEventKey *event,
@@ -806,6 +830,11 @@ ephy_node_view_button_press_cb (GtkWidget *treeview,
 	GtkTreeSelection *selection;
 	gboolean call_parent = TRUE, path_is_selected;
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+	
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (treeview)))
+	{
+		return GTK_WIDGET_CLASS (parent_class)->button_press_event (treeview, event);
+	}
 
 	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
 					   event->x,
@@ -843,6 +872,14 @@ ephy_node_view_button_press_cb (GtkWidget *treeview,
 
 			g_signal_emit_by_name (view, "popup_menu", &retval);
 		}
+		else if (event->button == 2)
+		{
+			EphyNode *clicked_node;
+			
+			clicked_node = process_middle_click (path, view);
+			g_signal_emit (G_OBJECT (view),
+				       ephy_node_view_signals[NODE_MIDDLE_CLICKED], 0, clicked_node);
+		}
 		else if (event->button == 1)
 		{
 			if (view->priv->toggle_column >= 0)
@@ -1278,6 +1315,12 @@ ephy_node_view_add_column (EphyNodeView *view,
 		gtk_tree_view_set_search_column (GTK_TREE_VIEW (view), column);
 		gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), TRUE);
 	}
+	
+	if (flags & EPHY_NODE_VIEW_ELLIPSIZED)
+	{
+		g_object_set (renderer, "ellipsize-set", TRUE,
+			      "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	}
 
 	if (ret != NULL)
 		*ret = gcolumn;
@@ -1373,7 +1416,7 @@ ephy_node_view_remove (EphyNodeView *view)
 	EphyNodeViewPrivate *priv = view->priv;
 	GList *list, *l;
 	EphyNode *node;
-	GtkTreeIter iter, iter2;
+	GtkTreeIter iter, iter2, iter3;
 	GtkTreePath *path;
 	GtkTreeRowReference *row_ref = NULL;
 	GtkTreeSelection *selection;
@@ -1388,9 +1431,9 @@ ephy_node_view_remove (EphyNodeView *view)
 
 	node = g_list_first (list)->data;
 	ephy_tree_model_node_iter_from_node (EPHY_TREE_MODEL_NODE (view->priv->nodemodel),
-					     node, &iter);
+					     node, &iter3);
 	gtk_tree_model_filter_convert_child_iter_to_iter (GTK_TREE_MODEL_FILTER (view->priv->filtermodel),
-							  &iter2, &iter);
+							  &iter2, &iter3);
 	gtk_tree_model_sort_convert_child_iter_to_iter (GTK_TREE_MODEL_SORT (view->priv->sortmodel),
 							&iter, &iter2);
 	iter2 = iter;
@@ -1708,6 +1751,16 @@ ephy_node_view_class_init (EphyNodeViewClass *klass)
 			      2,
 			      G_TYPE_POINTER,
 			      G_TYPE_POINTER);
+	ephy_node_view_signals[NODE_MIDDLE_CLICKED] =
+		g_signal_new ("node_middle_clicked",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyNodeViewClass, node_middle_clicked),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_POINTER);
 
 	g_type_class_add_private (object_class, sizeof (EphyNodeViewPrivate));
 }
