@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2003  David Bordoley
  *  Copyright © 2003-2004 Christian Persch
@@ -14,9 +15,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-tabs-menu.c 6588 2006-09-13 11:34:25Z chpe $
+ *  $Id: ephy-tabs-menu.c 7774 2007-12-16 12:55:20Z chpe $
  */
 
 #include "config.h"
@@ -26,6 +27,8 @@
 #include "ephy-marshal.h"
 #include "ephy-shell.h"
 #include "ephy-debug.h"
+#include "ephy-embed-container.h"
+#include "ephy-embed-utils.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtklabel.h>
@@ -37,11 +40,13 @@
 #include <gtk/gtkuimanager.h>
 #include <string.h>
 #include <stdlib.h>
-#include <libxml/entities.h>
 
 #define LABEL_WIDTH_CHARS 32
-#define ACTION_VERB_FORMAT		"JmpTab%x"
+#define ACTION_VERB_FORMAT_PREFIX       "JmpTab"
+#define ACTION_VERB_FORMAT_PREFIX_LEN   (6) /* strlen (ACTION_VERB_FORMAT_PREFIX) */
+#define ACTION_VERB_FORMAT		ACTION_VERB_FORMAT_PREFIX "%x"
 #define ACTION_VERB_FORMAT_LENGTH	strlen (ACTION_VERB_FORMAT) + 14 + 1
+#define ACTION_VERB_FORMAT_BASE         (16) /* %x is hex */
 #define ACCEL_PATH_FORMAT		"<Actions>/TabsActions/%s"
 #define ACCEL_PATH_FORMAT_LENGTH	strlen (ACCEL_PATH_FORMAT) -2 + ACTION_VERB_FORMAT_LENGTH
 #define DATA_KEY			"EphyTabsMenu::Action"
@@ -64,33 +69,88 @@ enum
 
 static void	ephy_tabs_menu_class_init	(EphyTabsMenuClass *klass);
 static void	ephy_tabs_menu_init	  	(EphyTabsMenu *menu);
+static void	ephy_tabs_menu_update		(EphyTabsMenu *menu);
 
-GType
-ephy_tabs_menu_get_type (void)
+/* FIXME: this can be severely optimised */
+static GByteArray *tabs_id_array = NULL;
+static guint n_tabs = 0;
+
+G_DEFINE_TYPE (EphyTabsMenu, ephy_tabs_menu, G_TYPE_OBJECT)
+
+/* We need to assign unique IDs to tabs, otherwise accels get confused in the
+ * tabs menu (bug #339548). We could use a serial #, but the ID is used in the
+ * action name which is stored in a GQuark and so we should allocate them
+ * efficiently.
+ */
+static guint
+allocate_tab_id (void)
 {
-	static GType type = 0;
+        int bit;
+        guint b, len;
+        guint8 *data;
+        guint8 byte, mask;
 
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyTabsMenuClass),
-			NULL, /* base_init */
-			NULL, /* base_finalize */
-			(GClassInitFunc) ephy_tabs_menu_class_init,
-			NULL,
-			NULL, /* class_data */
-			sizeof (EphyTab),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) ephy_tabs_menu_init
-		};
+        if (n_tabs++ == 0)
+        {
+                g_assert (tabs_id_array == NULL);
+                tabs_id_array = g_byte_array_sized_new (16);
+        }
 
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyTabsMenu",
-					       &our_info, 0);
-	}
+        /* Find a free ID */
+        len = tabs_id_array->len;
+        data = tabs_id_array->data;
+        for (b = 0; b < len; ++b)
+        {
+                if (data[b] != 0xff)
+                        break;
+        }
 
-	return type;
+        /* Need to append a new byte */
+        if (b == len)
+        {
+                guint8 bytes[] = { 0 };
+                g_byte_array_append (tabs_id_array, bytes, G_N_ELEMENTS (bytes));
+                g_assert (tabs_id_array->len > b);
+        }
+
+        data = tabs_id_array->data + b;
+        byte = 0xff ^ *data;
+        /* Now find the first free bit */
+        bit = g_bit_nth_lsf (byte, -1);
+        mask = 1 << bit;
+        g_assert (bit >= 0 && bit <= 7);
+        g_assert ((*data & mask) == 0);
+        /* And mark it as allocated */
+        *data |= mask;
+
+        return b * 8 + bit;
+}
+
+static void
+free_tab_id (GtkAction *action)
+{
+        const char *name;
+        guint id;
+        guint8 *data;
+        guint b, bit;
+
+        name = gtk_action_get_name (action);
+        id = g_ascii_strtoull (name + ACTION_VERB_FORMAT_PREFIX_LEN, NULL,
+                               ACTION_VERB_FORMAT_BASE);
+        g_assert (id >= 0 && id < tabs_id_array->len * 8);
+
+        b = id >> 3;
+        bit = id & 0x7;
+        data = tabs_id_array->data + b;
+        *data &= ~(1 << bit);
+
+        g_assert (n_tabs > 0);
+        if (--n_tabs == 0)
+        {
+                g_assert (tabs_id_array != NULL);
+                g_byte_array_free (tabs_id_array, TRUE);
+                tabs_id_array = NULL;
+        }
 }
 
 static void
@@ -98,39 +158,39 @@ tab_action_activate_cb (GtkToggleAction *action,
 			EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *priv = menu->priv;
-	EphyTab *tab;
+	EphyEmbed *embed;
 
 	if (gtk_toggle_action_get_active (action) == FALSE)
 	{
 		return;
 	}
 
-	tab = g_object_get_data (G_OBJECT (action), DATA_KEY);
-	g_return_if_fail (tab != NULL);
+	embed = g_object_get_data (G_OBJECT (action), DATA_KEY);
+	g_return_if_fail (embed != NULL);
 
-	LOG ("tab_action_activate_cb tab=%p", tab);
+	LOG ("tab_action_activate_cb embed=%p", embed);
 
-	if (ephy_window_get_active_tab (priv->window) != tab)
+	if (ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->window)) != embed)
 	{
-		ephy_window_jump_to_tab (priv->window, tab);
+                ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (priv->window), embed);
 	}
 }
 
 static void
-sync_tab_title (EphyTab *tab,
+sync_tab_title (EphyEmbed *embed,
 		GParamSpec *pspec,
 		GtkAction *action)
 {
 	const char *title;
 
-	title = ephy_tab_get_title_composite (tab);
+	title = ephy_embed_utils_get_title_composite (embed);
 
 	g_object_set (action, "label", title, NULL);
 }
 
 static void
 notebook_page_added_cb (EphyNotebook *notebook,
-			EphyTab *tab,
+			EphyEmbed *embed,
 			guint position,
 			EphyTabsMenu *menu)
 {
@@ -139,19 +199,18 @@ notebook_page_added_cb (EphyNotebook *notebook,
 	char verb[ACTION_VERB_FORMAT_LENGTH];
 	GSList *group;
 
-	LOG ("tab_added_cb tab=%p", tab);
+	LOG ("tab_added_cb embed=%p", embed);
 
-	g_snprintf (verb, sizeof (verb), ACTION_VERB_FORMAT,
-		    _ephy_tab_get_id (tab));
+	g_snprintf (verb, sizeof (verb), ACTION_VERB_FORMAT, allocate_tab_id ());
   
 	action = g_object_new (GTK_TYPE_RADIO_ACTION,
 			       "name", verb,
 			       "tooltip", _("Switch to this tab"),
 			       NULL);
 
-	sync_tab_title (tab, NULL, action);
+	sync_tab_title (embed, NULL, action);
 	/* make sure the action is alive when handling the signal, see bug #169833 */
-	g_signal_connect_object (tab, "notify::title",
+	g_signal_connect_object (embed, "notify::title",
 				 G_CALLBACK (sync_tab_title), action, 0);
 
 	gtk_action_group_add_action_with_accel (priv->action_group, action, NULL);
@@ -159,14 +218,14 @@ notebook_page_added_cb (EphyNotebook *notebook,
 	group = gtk_radio_action_get_group (GTK_RADIO_ACTION (priv->anchor_action));
 	gtk_radio_action_set_group (GTK_RADIO_ACTION (action), group);
 
-	/* set this here too, since tab-added comes after notify::active-tab */
-	if (ephy_window_get_active_tab (priv->window) == tab)
+	/* set this here too, since tab-added comes after notify::active-child */
+	if (ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->window)) == embed)
 	{
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 	}
 
-	g_object_set_data (G_OBJECT (tab), DATA_KEY, action);
-	g_object_set_data (G_OBJECT (action), DATA_KEY, tab);
+	g_object_set_data (G_OBJECT (embed), DATA_KEY, action);
+	g_object_set_data (G_OBJECT (action), DATA_KEY, embed);
 
 	g_signal_connect (action, "activate",
 			  G_CALLBACK (tab_action_activate_cb), menu);
@@ -178,25 +237,27 @@ notebook_page_added_cb (EphyNotebook *notebook,
 
 static void
 notebook_page_removed_cb (EphyNotebook *notebook,
-			  EphyTab *tab,
+			  EphyEmbed *embed,
 			  guint position,
 			  EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *priv = menu->priv;
 	GtkAction *action;
 													     
-	LOG ("tab_removed_cb tab=%p", tab);
+	LOG ("tab_removed_cb embed=%p", embed);
 
-	action = g_object_get_data (G_OBJECT (tab), DATA_KEY);
+	action = g_object_get_data (G_OBJECT (embed), DATA_KEY);
 	g_return_if_fail (action != NULL);
 
+        free_tab_id (action);
+
 	g_signal_handlers_disconnect_by_func
-		(tab, G_CALLBACK (sync_tab_title), action);
+		(embed, G_CALLBACK (sync_tab_title), action);
 
 	g_signal_handlers_disconnect_by_func
 		(action, G_CALLBACK (tab_action_activate_cb), menu);
 
-	g_object_set_data (G_OBJECT (tab), DATA_KEY, NULL);
+	g_object_set_data (G_OBJECT (embed), DATA_KEY, NULL);
  	gtk_action_group_remove_action (priv->action_group, action);
 
 	ephy_tabs_menu_update (menu);
@@ -204,7 +265,7 @@ notebook_page_removed_cb (EphyNotebook *notebook,
 
 static void
 notebook_page_reordered_cb (EphyNotebook *notebook,
-			    EphyTab *tab,
+			    EphyEmbed *embed,
 			    guint position,
 			    EphyTabsMenu *menu)
 {
@@ -236,18 +297,16 @@ sync_active_tab (EphyWindow *window,
 		 GParamSpec *pspec,
 		 EphyTabsMenu *menu)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 	GtkAction *action;
 
-	tab = ephy_window_get_active_tab (window);
+	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+	if (embed == NULL) return;
 
-	LOG ("active tab is tab %p", tab);
- 
-	g_return_if_fail (tab != NULL);
-	if (tab == NULL) return;
+	LOG ("active tab is embed %p", embed);
 
-	action = g_object_get_data (G_OBJECT (tab), DATA_KEY);
-	/* happens initially, since the ::active-tab comes before
+	action = g_object_get_data (G_OBJECT (embed), DATA_KEY);
+	/* happens initially, since the ::active-child comes before
 	* the ::tab-added signal
 	*/
 	/* FIXME that's not true with gtk+ 2.9 anymore */
@@ -279,7 +338,7 @@ ephy_tabs_menu_set_window (EphyTabsMenu *menu,
 	g_signal_connect (priv->action_group, "connect-proxy",
 			  G_CALLBACK (connect_proxy_cb), NULL);
 
-	g_signal_connect (window, "notify::active-tab",
+	g_signal_connect (window, "notify::active-child",
 			  G_CALLBACK (sync_active_tab), menu);
 
 	notebook = ephy_window_get_notebook (window);
@@ -327,14 +386,15 @@ ephy_tabs_menu_class_init (EphyTabsMenuClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_WINDOW,
-					 g_param_spec_object ("window",
-							      "Window",
-							      "Parent window",
+					 g_param_spec_object ("window", NULL, NULL,
 							      EPHY_TYPE_WINDOW,
-							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
+							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
 							      G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof (EphyTabsMenuPrivate));
+
+	/* We don't want to save accels, so skip them */
+	gtk_accel_map_add_filter ("<Actions>/TabsActions/JmpTab*");
 }
 
 static void
@@ -406,7 +466,7 @@ tab_set_action_accelerator (GtkActionGroup *action_group,
 	}
 }
 
-void
+static void
 ephy_tabs_menu_update (EphyTabsMenu *menu)
 {
 	EphyTabsMenuPrivate *p = menu->priv;
@@ -422,7 +482,7 @@ ephy_tabs_menu_update (EphyTabsMenu *menu)
 
 	ephy_tabs_menu_clean (menu);
 
-	tabs = ephy_window_get_tabs (p->window);
+	tabs = ephy_embed_container_get_children (EPHY_EMBED_CONTAINER (p->window));
 
 	if (g_list_length (tabs) == 0) return;
 
